package cmt.test;

import junit.framework.TestCase;

import javax.ejb.EJB;
import javax.ejb.EJBException;
import javax.ejb.EJBTransactionRequiredException;
import javax.ejb.embeddable.EJBContainer;

import cmt.Movie;
import cmt.Movies;
import cmt.MoviesMandatory;
import cmt.MoviesRequired;
import cmt.MoviesSupports;


import java.util.List;
import java.util.Properties;
import java.util.concurrent.Callable;

public class MoviesTest extends TestCase {

    @EJB
    private MoviesMandatory moviesM;
    
    @EJB
    private MoviesSupports moviesS;
    
    @EJB
    private MoviesRequired moviesR;

    @EJB
    private Caller transactionalCaller;

    protected void setUp() throws Exception {
        final Properties p = new Properties();
        p.put("movieDatabase", "new://Resource?type=DataSource");
        p.put("movieDatabase.JdbcDriver", "org.hsqldb.jdbcDriver");
        p.put("movieDatabase.JdbcUrl", "jdbc:hsqldb:mem:moviedb");

        EJBContainer.createEJBContainer(p).getContext().bind("inject", this);
    }

    private void actions(Movies m) throws Exception{
    	m.addMovie(new Movie("Quentin Tarantino", "Reservoir Dogs", 1992));
        m.addMovie(new Movie("Joel Coen", "Fargo", 1996));
        m.addMovie(new Movie("Joel Coen", "The Big Lebowski", 1998));

        List<Movie> list = m.getMovies();
        assertEquals("List.size()", 3, list.size());

        for (Movie movie : list) {
        	m.deleteMovie(movie);
        }
        assertEquals("MoviesMandatory.getMovies()", 0, m.getMovies().size());
    }
    
    private void mandatoryWithTransaction() throws Exception {
    	actions(moviesM);
    }
    
    private void supportsNoTransaction() throws Exception {
    	moviesS.MethodWithoutTransactionRequirements();
    }
    
    private void supportsWithTransaction() throws Exception {
    	actions(moviesS);
    }

    private void requiredWithTransaction() throws Exception {
    	actions(moviesR);
    }

	public void testWithTransaction() throws Exception {
        transactionalCaller.call(new Callable<Object>() {
            public Object call() throws Exception {
            	mandatoryWithTransaction();
            	supportsNoTransaction();
            	supportsWithTransaction();
            	requiredWithTransaction();
                return null;
            }
        });
    }

    public void testWithoutTransaction() throws Exception {

//    	MANDATORY
    	try {
    		mandatoryWithTransaction();
            fail("The MoviesMandatory bean should be using TransactionAttributeType.MANDATORY");
        } catch (EJBTransactionRequiredException e) {
            // good, our MoviesMandatory bean is using MANDATORY as we want
        }
    	
//    	SUPPORTED
    	supportsNoTransaction();
    	//method doesn't contain methods that need cmt so everything is all right   	
    	try {
    		supportsWithTransaction();
    		fail("The MoviesSupports bean should be using failing on persist()");
    	}
    	catch (EJBException e) {
            // good beacause MoviesSupports bean is using SUPPORTS so it will be allowed to run but will fail on persist(movie)
        }
    	
//    	REQUIRED
    	requiredWithTransaction();
    	// Required will create its own cmt so it should work (with some changes)
    }
}
